require 'spec_helper'
require 'middleman_spec_helper'

describe BranchableCDNAssets::Middleman::Helpers do


  describe "with a single instance" do
    before :each do
      Given.fixture 'base'
      Given.file 'cdn/master.manifest',      "image_one\nimage_all\nimage_remote_one\nimg.one.jpg\nimage_prod"
      Given.file 'cdn/some_branch.manifest', "image_one\nimage_all\nimage_remote_two\n"
      Given.file 'cdn/image_one', ''
      Given.file 'cdn/image_two', ''
      Given.file 'cdn/image_all', ''
      Given.file 'cdn/image_prod', ''

      @mm = Middleman::Fixture.app
      @extension = @mm.extensions[:cdn_assets].first[1]

      # split test based on changes in v4
      if @mm.respond_to?(:template_context_class)
        @context = @mm.template_context_class.new(@mm)
      else
        @context = @mm
      end
    end

    after :each do
      Given.cleanup!
    end

    describe "sitemap helpers" do
      describe "#cdn_asset_url" do
        context "when asset is on production cdn" do
          it "returns the 'live' url of the asset" do
            allow( Asgit ).to receive(:current_branch).and_return('master')
            expect( @context.cdn_asset_url('image_remote_one') ).to eq 'http://production.com/image_remote_one'
          end
        end

        context "when asset is on staging cdn" do
          it "returns the 'live' url of the asset" do
            allow( Asgit ).to receive(:current_branch).and_return('some_branch')
            expect( @context.cdn_asset_url('image_remote_two') ).to eq 'http://staging.com/some_branch/image_remote_two'
          end
        end

        context "when asset is local" do
          it "returns the 'localhost' url of the asset" do
            expect( @context.cdn_asset_url('image_two') ).to eq "/assets/cdn/#{@extension.id}/image_two"
          end
        end

        context "when asset is local and listed in a manifest" do
          it "returns the 'localhost' url of the asset" do
            expect( @context.cdn_asset_url('image_one') ).to eq "/assets/cdn/#{@extension.id}/image_one"
          end

          context "when building" do
            it "returns remote url" do
              @mm.config[:mode] = :build

              allow( Asgit ).to receive(:current_branch).and_return('some_branch')
              expect( @context.cdn_asset_url('image_all') ).to eq "http://staging.com/some_branch/image_all"
              expect( @context.cdn_asset_url('image_one') ).to eq "http://staging.com/some_branch/image_one"
              expect( @context.cdn_asset_url('image_prod') ).to eq "http://production.com/image_prod"
            end
          end
        end

        context "when the asset is missing" do
          it "raises an error with the asset name" do
            expect{
              @context.cdn_asset_url('missing_image')
            }.to raise_error BranchableCDNAssets::Middleman::FileNotFoundError, "missing asset at 'missing_image'"
          end
        end

        context "when assets have multiple extensions" do
          it "returns the url with the given extensions" do
            allow( Asgit ).to receive(:current_branch).and_return('master')
            expect( @context.cdn_asset_url('img.one.jpg') ).to eq "http://production.com/img.one.jpg"
          end
          it "raises an error if the root asset is missing" do
            expect{
              @context.cdn_asset_url('missing_image.1.jpg')
            }.to raise_error BranchableCDNAssets::Middleman::FileNotFoundError, "missing asset at 'missing_image.1.jpg'"
          end
        end

        context "with added file extensions" do
          it "returns the url with the given extensions" do
            allow( Asgit ).to receive(:current_branch).and_return('master')
            expect( @context.cdn_asset_url('image_one?#iefix') ).to eq "/assets/cdn/#{@extension.id}/image_one?#iefix"
            expect( @context.cdn_asset_url('img.one.jpg?#iefix') ).to eq "http://production.com/img.one.jpg?#iefix"
          end
          it "raises an error if the root asset is missing" do
            expect{
              @context.cdn_asset_url('missing_image?#iefix')
            }.to raise_error BranchableCDNAssets::Middleman::FileNotFoundError, "missing asset at 'missing_image'"
            expect{
              @context.cdn_asset_url('missing_image.one.jpg?#iefix')
            }.to raise_error BranchableCDNAssets::Middleman::FileNotFoundError, "missing asset at 'missing_image.one.jpg'"
          end
        end
      end
    end

    describe "sass helpers" do
      describe "#cdn_asset_url" do
        it "calls mm#cdn_asset_url with the asset path" do
          expect( @context ).to receive(:cdn_asset_url).with('image_path', nil)

          source = 'body { background: url( cdn_asset_url("image_path") ); }'
          Sass.compile( source, {custom: {middleman_context: @context}} )
        end
      end
    end
  end


  describe "with a multiple instances" do
    before :all do
      Given.fixture 'multi'
      Given.file 'cdn/master.manifest', "image_one\nimage_remote_cdn1"
      Given.file 'cdn/image_local', ''
      Given.file 'cdn/image_cdn', ''
      Given.file 'cdn2/master.manifest', "image_one\nimage_remote_cdn2"
      Given.file 'cdn2/image_local', ''
      Given.file 'cdn2/image_cdn2', ''

      @mm = Middleman::Fixture.app
      @cdn1 = @mm.extensions[:cdn_assets]["instance_0"]
      @cdn2 = @mm.extensions[:cdn_assets]["instance_1"]

      # split test based on changes in v4
      if @mm.respond_to?(:template_context_class)
        @context = @mm.template_context_class.new(@mm)
      else
        @context = @mm
      end
    end

    after :all do
      Given.cleanup!
    end


    describe "sitemap helpers" do
      describe "#cdn_asset_url" do
        context "when file exists on only one cdn" do
          it "finds remote files on correct cdn" do
            allow( Asgit ).to receive(:current_branch).and_return('master')

            expect( @context.cdn_asset_url('image_remote_cdn1') ).to eq 'http://production.com/image_remote_cdn1'
            expect( @context.cdn_asset_url('image_remote_cdn2') ).to eq 'http://production.com/image_remote_cdn2'
          end

          it "finds local files on correct cdn" do
            expect( @context.cdn_asset_url('image_cdn') ).to  eq "/assets/cdn/cdn1/image_cdn"
            expect( @context.cdn_asset_url('image_cdn2') ).to eq "/assets/cdn/cdn2/image_cdn2"
          end
        end

        context "when file exists on multiple cdns" do
          it "returns file on given cdn" do
            expect( @context.cdn_asset_url('image_local', 'cdn1') ).to eq "/assets/cdn/cdn1/image_local"
            expect( @context.cdn_asset_url('image_local', 'cdn2') ).to eq "/assets/cdn/cdn2/image_local"
          end

          it "raises error if a cdn key isn't given" do
            expect{
              @context.cdn_asset_url('image_local')
            }.to raise_error BranchableCDNAssets::Middleman::MultipleCDNError
          end
        end
      end
    end

    describe "sass helpers" do
      describe "#cdn_asset_url" do
        it "is called with the cdn argument if given" do
          expect( @context ).to receive(:cdn_asset_url).with('image_path', 'cdn2')

          source = 'body { background: url( cdn_asset_url("image_path", "cdn2") ); }'
          Sass.compile( source, {custom: {middleman_context: @context}} )
        end

        it "is called with SassC as well" do
          expect( @context ).to receive(:cdn_asset_url).with('image_path', 'cdn2')

          source = 'body { background: url( cdn_asset_url("image_path", "cdn2") ); }'
          SassC::Engine.new(source, {custom: {middleman_context: @context}}).render
        end
      end
    end

  end
end